/******************************************************************************/
/*                                                                            */
/* Version 1.11                                                               */
/* Author: Arjan van Vught                 Date: March     2003               */
/*                                                                            */
/******************************************************************************/
/* Version 1.0   First release NT Get environment variable                    */
/* Version 1.1   Get ALL environment variables                                */
/* Version 1.2   Code clean uo - removed debug info                           */
/* Version 1.3   Updated for MQSeries Integrator 2.0.1                        */
/* Version 1.4   First release AIX                                            */
/* Version 1.5   Convert option added                                         */
/* Version 1.6   AIX: Bug empty variable fixed                                */
/* Version 1.7   First release Sun Solaris                                    */
/*               HP-UX Ready - Not Tested -                                   */
/* Version 1.8   Improved debug info for Sun Solaris                          */
/* Version 1.9   First release Linux                                          */
/*               Sun Solaris, Linux: Bug empty variable fixed                 */
/* Version 1.10  First release HP-UX                                          */
/* Version 1.11  Updated for WebSphere MQ Integrator 2.1                      */
/******************************************************************************/
/* Implementation Notes:                                                      */
/*                                                                            */
/*   Portability:                                                             */
/*                                                                            */
/*     This code is intended to be portable and run on any of the operating   */
/*     systems supported by WebSphere MQ Integrator.                          */
/*                                                                            */
/******************************************************************************/
/* Includes */
/* <malloc.h> is not available on /390 */
#ifndef __MVS__
  #include <malloc.h>
#endif
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <wchar.h>

#include "GetEnvPlugIn.h"

/******************************************************************************/
/*                                                                            */
/* Plugin Node Utility function:                                              */
/*                                                                            */
/*  Acquire storage for attribute and link to the context                     */
/*                                                                            */
/******************************************************************************/
ATTRIBUTE_TBL_ENTRY *insertAttributeTableEntry(NODE_CONTEXT_ST *context, CciChar *nodeName, CciChar *attrName, int type)
{
   /*@unused@*/static char *functionName = (char *)"insertAttributeTableEntry()";/* 28/07/2001 */

   ATTRIBUTE_TBL_ENTRY *entry = NULL;
   int attrSize = 4;																				/* 28/07/2001 */

   /* Allocate storage for the attribute entry */
   entry = malloc(sizeof(ATTRIBUTE_TBL_ENTRY));

   if (entry) {
      entry->next = 0;
      entry->type = type;

      switch (entry->type) {
         case CNI_TYPE_INTEGER:
            attrSize = 4;
            break;
         case CNI_TYPE_STRING:
            attrSize = MAX_STRING_SIZE;
         default:
            /* Error: Invalid datatype */
            break;
       }

       /* Acquire and clear storage for attribute value */
       entry->valptr = malloc(attrSize);
       if (entry->valptr) memset(entry->valptr, 0, attrSize);

       /* Store the attribute name */
       CciCharCpy(entry->name, attrName);

       /* Link an existing previous element to this one */
       if (context->attrListPrevious) context->attrListPrevious->next = entry;
       else if ((context->attrListHead) == 0) context->attrListHead = entry;

       /* Save the pointer to the previous element */
       context->attrListPrevious = entry;
   }
   else {
      /* Error: Unable to allocate memory */
   }

   return(entry);

}

/******************************************************************************/
/*                                                                            */
/* Plugin Node Utility function:                                              */
/*                                                                            */ 
/*  Acquire storage for an input terminal entry and link it to the context    */
/*                                                                            */
/******************************************************************************/
TERMINAL_LIST_ENTRY *insertInputTerminalListEntry(NODE_CONTEXT_ST *context, CciChar *terminalName)
{
   /*@unused@*/static char *functionName = (char *)"insertInputTerminalListEntry()";/* 28/07/2001 */

   TERMINAL_LIST_ENTRY *entry = NULL;
   int rc;

   entry = (TERMINAL_LIST_ENTRY *)malloc(sizeof(TERMINAL_LIST_ENTRY));

   if (entry) {

      /* This entry is the current end of the list */
      entry->next = 0;

      /* Store the terminal name */
      CciCharCpy(entry->name, terminalName);

      /* Create terminal and save its handle */
      entry->handle = cniCreateInputTerminal(&rc, context->nodeObject, (CciChar*)terminalName);

      /* Link an existing previous element to this one */
      if (context->inputTerminalListPrevious) context->inputTerminalListPrevious->next = entry;
      else if ((context->inputTerminalListHead) == 0) context->inputTerminalListHead = entry;

      /* Save the pointer to the previous element */
      context->inputTerminalListPrevious = entry;
   }
   else {
    /* Error: Unable to allocate memory */
   }

   return(entry);
}

/******************************************************************************/
/*                                                                            */
/* Plugin Node Utility function:                                              */
/*                                                                            */ 
/*  Acquire storage for an output terminal and link it to the context         */
/*                                                                            */
/******************************************************************************/
TERMINAL_LIST_ENTRY *insertOutputTerminalListEntry(NODE_CONTEXT_ST *context, CciChar *terminalName)
{
   /*@unused@*/static char *functionName = (char *)"insertOutputTerminalListEntry()";/* 28/07/2001 */

   TERMINAL_LIST_ENTRY *entry = NULL;
   int rc = 0;

   entry = (TERMINAL_LIST_ENTRY *)malloc(sizeof(TERMINAL_LIST_ENTRY));
   
   if (entry) {

      /* This entry is the current end of the list */
      entry->next = 0;

      /* Store the terminal name */
      CciCharCpy(entry->name, terminalName);

      /* Create terminal and save its handle */
      entry->handle = cniCreateOutputTerminal(&rc, context->nodeObject, (CciChar*)terminalName);

      /* Link an existing previous element to this one */
      if (context->outputTerminalListPrevious) context->outputTerminalListPrevious->next = entry;
      else if ((context->outputTerminalListHead) == 0) context->outputTerminalListHead = entry;

      /* Save the pointer to the previous element */
      context->outputTerminalListPrevious = entry;
   }
   else {
      /* Error: Unable to allocate memory */
   }

   return(entry);
}

/******************************************************************************/
/*                                                                            */
/* Plugin Node Utility function:                                              */
/*                                                                            */ 
/*  Return the terminal object for a named input terminal                     */
/*                                                                            */
/******************************************************************************/
void *getInputTerminalHandle(NODE_CONTEXT_ST *context, CciChar *terminalName)
{
   /*@unused@*/static char *functionName = (char *)"getInputTerminalHandle()";/* 28/07/2001 */

   TERMINAL_LIST_ENTRY *p = ((NODE_CONTEXT_ST *)context)->inputTerminalListHead;

   /* Scan the list for a matching terminal name */
   while (p != 0) {
      if (CciCharCmp(p->name, terminalName) == 0) {
         /* Match found; return it */
         return(p->handle);
      }
      p = (TERMINAL_LIST_ENTRY *)p->next;
   }

   /* No match was found */
   return(0);
}

/******************************************************************************/
/*                                                                            */
/* Plugin Node Utility function:                                              */
/*                                                                            */ 
/*  Return the terminal object for a named output terminal                    */
/*                                                                            */
/******************************************************************************/
void *getOutputTerminalHandle(NODE_CONTEXT_ST *context, CciChar *terminalName)
{
   /*@unused@*/static char *functionName = (char *)"getOutputTerminalHandle()";/* 28/07/2001 */

   TERMINAL_LIST_ENTRY *p = ((NODE_CONTEXT_ST *)context)->outputTerminalListHead;

   /* Scan the list for a matching terminal name */
   while (p != 0) {
      if (CciCharCmp(p->name, terminalName) == 0) {

         /* Match found; return it */
         return(p->handle);
      }
      p = (TERMINAL_LIST_ENTRY *)p->next;
   }

   /* No match was found */
   return(0);
}



/******************************************************************************/
/* Node Implementation Functions                                              */
/*                                                                            */
/* These are mandatory and MUST be implemented by the programmer.             */
/*                                                                            */
/* createNodeContext                                                          */
/* deleteNodeContext                                                          */
/* evaluate                                                                   */
/* getAttribute                                                               */
/* getAttriubteName                                                           */
/* setAttribute                                                               */
/* bipGetMessageflowNodeFactory                                               */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/*                                                                            */
/* Plugin Node Implementation Function:                cniCreateNodeContext() */
/*                                                                            */
/******************************************************************************/
CciContext *createNodeContext(CciFactory *factoryObject, CciChar *nodeName, CciNode *nodeObject)
{
   /*@unused@*/static char  *functionName = (char *)"createNodeContext()";		/* 28/07/2001 */
   NODE_CONTEXT_ST *p = NULL;

   /* Allocate a pointer to the local context */
   p = malloc(sizeof(NODE_CONTEXT_ST));

   /* If malloc was successful */
   if (p) {
      /* Clear the context area */
      memset(p, 0, sizeof(NODE_CONTEXT_ST)); /* Clear the context area */

      /* Save our node object pointer in our context */
	   p->nodeObject = nodeObject;

      /* Save our node name */
		CciCharNCpy((CciChar *)&p->nodeName, nodeName, MAX_NODE_NAME_LEN);

		/* Create attributes and set default values */
		{
			const CciChar* ucsAttr = CciString("Convert") ;
			insertAttributeTableEntry(p, nodeName, (CciChar*)ucsAttr, CNI_TYPE_STRING);
			free((void *)ucsAttr) ;
		}

      /* Create terminals */
      {
         const CciChar* ucsIn = CciString("in") ;
         insertInputTerminalListEntry(p, (CciChar*)ucsIn);
         free((void *)ucsIn) ;
      }
  
      {
         const CciChar* ucsOut = CciString("out") ;
         insertOutputTerminalListEntry(p, (CciChar*)ucsOut);
         free((void *)ucsOut) ;
      }

      {
         const CciChar* ucsFailure = CciString("failure") ;
         insertOutputTerminalListEntry(p, (CciChar*)ucsFailure);
         free((void *)ucsFailure) ;
      }
   }
   else {
      /* Error handling if malloc failed */
   }

   /* Return the pointer to the local context */
   return((CciContext*)p);
}


/******************************************************************************/
/*                                                                            */
/* Plugin Node Implementation Function:                cniDeleteNodeContext() */
/*                                                                            */
/* This function is called when an instance of a node is deleted.             */
/*                                                                            */
/******************************************************************************/
void deleteNodeContext(CciContext* context)
{
  /*@unused@*/static char *functionName = (char *)"deleteNodeContext()";		/* 28/07/2001 */

  free((NODE_CONTEXT_ST *)context);

  return;
}


/******************************************************************************/
/*                                                                            */
/* Plugin Node Implementation Function:                     cniGetAttribute() */
/*                                                                            */
/******************************************************************************/
int getAttribute(CciContext *context, CciChar *attrName, CciChar *buffer, int bufsize)
{
   /*@unused@*/static char *functionName = (char *)"getAttribute()";				/* 28/07/2001 */

   ATTRIBUTE_TBL_ENTRY *p = ((NODE_CONTEXT_ST *)context)->attrListHead;

   char    string[256];  

   while (p != 0) {
		if (CciCharCmp(p->name, attrName) == 0) {
         switch(p->type) {
            case CNI_TYPE_INTEGER:
               sprintf((char*)&string, "%d", *((int*)p->valptr));
					{
						const CciChar* ucsAttrValue = CciString(string);
						CciCharNCpy(buffer, ucsAttrValue, min(bufsize, 256));
						free((void *)ucsAttrValue);
					}
               break;
            case CNI_TYPE_STRING:
					{
						const CciChar* ucsAttrValue = CciString((char *)p->valptr);
						CciCharNCpy(buffer, ucsAttrValue, min(MAX_STRING_SIZE, bufsize));
						free((void *)ucsAttrValue);
					}
					break;																			/* 28/07/2001 */
            default:
            /* Error: Invalid datatype */
            break;
         }

         /* Normal return; attribute value returned */ 
         return(0);
      }

      p = p->next;
   }

   /* Attribute not owned by plugin */
   return(1); 
}

/******************************************************************************/
/*                                                                            */
/* Plugin Node Implementation Function:                     cniSetAttribute() */
/*                                                                            */
/******************************************************************************/
int setAttribute(CciContext *context, CciChar *attrName, CciChar *attrValue)
{
   /*@unused@*/static char *functionName = (char *)"setAttribute()";				/* 28/07/2001 */

   ATTRIBUTE_TBL_ENTRY *p = ((NODE_CONTEXT_ST *)context)->attrListHead;
   
   while (p != 0) {
      if (CciCharCmp(p->name, attrName) == 0) {
      switch(p->type) {
         case CNI_TYPE_INTEGER:
            {
               const char* mbAttrValue = mbString(attrValue);
               *((int *)p->valptr) = atoi(mbAttrValue);
               free((void *)mbAttrValue);
            }
            break;
         case CNI_TYPE_STRING:
            {
               const char* mbAttrValue = mbString(attrValue);
               int len = strlen(mbAttrValue);
               if(len > MAX_STRING_SIZE)
                  len = MAX_STRING_SIZE;
               memcpy(p->valptr, mbAttrValue, len);
               free((void *)mbAttrValue);
            }
            break;

         default:
            /* Error: Invalid datatype */
             break;
         }

         /* Normal return; attribute value stored by plugin */
         return(0);
      }

      p = (ATTRIBUTE_TBL_ENTRY *)p->next;
   }

   /* Attribute not owned by plugin */
   return(1);
}

/******************************************************************************/
/*                                                                            */
/* Plugin Node Implementation Function:                     cniGetAttribute() */
/*                                                                            */
/******************************************************************************/
int getAttributeName(CciContext* context, int index, CciChar* buffer, int bufferSize)
{
   /*@unused@*/static char *functionName = (char *)"getAttributeName()";		/* 28/07/2001 */

   ATTRIBUTE_TBL_ENTRY *p = ((NODE_CONTEXT_ST *)context)->attrListHead;
   int count = 0;
    
   while (p != 0) {
      if (count == index) {
         CciCharCpy(buffer, p->name);
         /* Normal return; attribute name returned */
         return(0);
      }
      count++;
      p = (ATTRIBUTE_TBL_ENTRY *)p->next;
   }

   /* Attribute not owned by the plugin node; defer to base */
   return(1);
}


/******************************************************************************/
/*                                                                            */
/* Plugin Node Implementation Function:                         cniEvaluate() */
/*                                                                            */
/******************************************************************************/
void evaluate(CciContext *context, CciMessage *destinationList, CciMessage *exceptionList, CciMessage *message)
{
   /*@unused@*/static char *functionName = (char *)"evaluate()";					/* 28/07/2001 */
   CCI_EXCEPTION_ST exception_st;
      
   int rc            = 0;
   int retvalue      = 1;
   int EqualOffset   = 0;

   char **search     = _environ;

   CciChar *wName    = NULL;
   CciChar *wValue   = NULL;

   CciChar AttributeBuffer[256];
       
   CciTerminal *outputTerminalObject = NULL;

   CciElement *rootElement    = NULL;
   CciElement *newElement     = NULL;
   CciElement *bodyChild      = NULL;
   CciElement *outRootElement = NULL;

   CciMessage  *outMsg = NULL;

   char ConvertChoice  = CONVERTNO;

#ifdef _DEBUG
   FILE *outputFile    = NULL;

   /* Open the debug file for output */
#if defined(__CCI_LINUX__)																		/* 16/09/2001*/
   outputFile   = fopen("/tmp/GetEnvDebug.txt", "w");									/* 16/09/2001*/
   fprintf(outputFile, "GetEnvPlugIn for MQSeries Integrator V2 for Linux\n");/* 16/09/2001*/
#elif defined(__CCI_SOLARIS__)
   outputFile   = fopen("/tmp/GetEnvDebug.txt", "w");
   fprintf(outputFile, "GetEnvPlugIn for MQSeries Integrator V2 for Solaris\n");
#elif defined(__CCI_HPUX__)
   outputFile   = fopen("/tmp/GetEnvDebug.txt", "w");
   fprintf(outputFile, "GetEnvPlugIn for MQSeries Integrator V2 for HP-UX\n");
#elif defined(__CCI_AIX__)
   outputFile   = fopen("/tmp/GetEnvDebug.txt", "w");
   fprintf(outputFile, "GetEnvPlugIn for MQSeries Integrator V2 for AIX\n");
#else
   outputFile   = fopen("c:\\GetEnvDebug.txt", "w");
   fprintf(outputFile, "GetEnvPlugIn for MQSeries Integrator V2 for Windows NT/2000\n");
#endif
   fflush(outputFile);
#endif

   /* Get attribute value of "Convert" attribute */
	rc = getAttribute(context, (CciChar *)constConvert, AttributeBuffer, sizeof(AttributeBuffer));

#ifdef _DEBUG
	fprintf(outputFile, "rc = getAttribute %d\n", rc);
	{
		const char *mbAttributeBuffer = mbString(AttributeBuffer);
		fprintf(outputFile, "AttributeBuffer = %s\n\n", mbAttributeBuffer);
		free((void *)mbAttributeBuffer);
	}
   fflush(outputFile);
#endif

   if (CciCharCmp(AttributeBuffer, (CciChar *)constLowerCase) == 0)
      ConvertChoice = CONVERTTOLOWER;
   else if (CciCharCmp(AttributeBuffer, (CciChar *)constUpperCase) == 0)
      ConvertChoice = CONVERTTOUPPER;
   else
      ConvertChoice = CONVERTNO;

#ifdef _DEBUG
   fprintf(outputFile, "Convert = %d\n\n", ConvertChoice);
   fflush(outputFile);
#endif
   
   /* Get the root element */
   rootElement = cniRootElement(&rc, message);
   CCI_CHECK_RC();

   if (search) {
      /* Create a new message for output */
      outMsg = cniCreateMessage(&rc, cniGetMessageContext(&rc, message));
      CCI_CHECK_RC();

      /* Get the root element of the output message */
      outRootElement = cniRootElement(&rc, outMsg);
      CCI_CHECK_RC();

      /* Copy the contents of the input message to the output message */
      cniCopyElementTree(&rc, rootElement, outRootElement);
      CCI_CHECK_RC();

      bodyChild = cniCreateElementAsLastChild(&rc, outRootElement);
      CCI_CHECK_RC();
      cniSetElementType(&rc, bodyChild, CCI_ELEMENT_TYPE_NAME);
      CCI_CHECK_RC();
		cniSetElementName(&rc, bodyChild, (CciChar *)constENV);
      CCI_CHECK_RC();
   
      if (search) {
         while (*search) {

            wgetNameValue(*search, &wName, &wValue);

            switch(ConvertChoice) {
               case(CONVERTTOLOWER):
                  wStrLower(wName);
                  break;
               case(CONVERTTOUPPER):
                  wStrUpper(wName);
                  break;
               default:
                  break;
            }

            newElement = cniCreateElementAsLastChild(&rc, bodyChild);
            CCI_CHECK_RC();
            cniSetElementType(&rc, newElement, CCI_ELEMENT_TYPE_NAME_VALUE);
            CCI_CHECK_RC();
            cniSetElementName(&rc, newElement, wName);
            CCI_CHECK_RC();
            /* 5 Feb 2001 BEGIN */
            if (wValue == NULL)
               cniSetElementCharacterValue(&rc, newElement, (const CciChar *)constSpace, CciCharLen(constSpace));/* 28/07/2001 */
            else {
               cniSetElementCharacterValue(&rc, newElement, (const CciChar *)wValue, CciCharLen(wValue));
		
            }
            /* 5 Feb 2001 END */
            CCI_CHECK_RC();

            free(wName);
            if (wValue != NULL)
               free(wValue);

            search++;

         } /* while (*search) */

      } /* if (search) */

      /* Get handle of output terminal */
      outputTerminalObject = getOutputTerminalHandle(context, (CciChar*)constOut);
      message = outMsg;
   } /* if (search) */
   else
      outputTerminalObject = getOutputTerminalHandle(context, (CciChar*)constFailure);

#ifdef _DEBUG
   /* close the DEBUG output file */
   fclose(outputFile);
#endif

   /* If the terminal exists and is attached, propagate to it */
   if (outputTerminalObject) {
      if (cniIsTerminalAttached(&rc, outputTerminalObject)) {
         if (rc == CCI_SUCCESS) {
            retvalue = cniPropagate(&rc, outputTerminalObject, destinationList, exceptionList, message);
            if (rc != CCI_SUCCESS) {
               if (rc == CCI_EXCEPTION) {
                  /* Get details of the exception */
                  memset(&exception_st, 0, sizeof(exception_st));
                  cciGetLastExceptionData(&rc, &exception_st);
                  /* Any local error handling may go here */

                  /* We must "rethrow" the exception; note this does not return */
                  cciRethrowLastException(&rc);
               }
               /* Some other error...the plugin might choose to log it using the CciLog() */
               /* utility function                                                        */
            }
         }
         else {
            if (rc == CCI_EXCEPTION) {
               /* We must "rethrow" the exception; note this does not return */
               cciRethrowLastException(&rc);
            }
            else {
               /* Some other error...the plugin might choose to log it using the CciLog() */
               /* utility function                                                        */
            }
         }
      }
   }
   else {
      /* Terminal did not exist...severe internal error...the plugin might choose to */
      /* log this situation using the cciLog() utility function.                     */
   }

   if (outMsg != NULL) {
      /* Delete the message we created now we have finished with it */
      cniDeleteMessage(0, outMsg);
  }

   return;
}

/******************************************************************************/
/*                                                                            */
/* Plugin Node Utility function:                                              */
/*                                                                            */
/*  initNodeConstants                                                         */
/*                                                                            */
/*  Creates Cci constant strings from defined char * constant strings.        */
/*                                                                            */
/******************************************************************************/
void initNodeConstants(void)
{
  constMQSeriesIntegrator2 = CciString(CONST_MQSERIES_INTEGRATOR_2);
  constPlugInNode				= CciString(CONST_PLUGIN_NODE);
  constPluginNodeFactory	= CciString(CONST_PLUGIN_NODE_FACTORY);
  constOut		            = CciString(CONST_OUT);
  constFailure             = CciString(CONST_FAILURE);
  constENV                 = CciString(CONST_ENV);
  constConvert             = CciString(CONST_CONVERT);
  constUpperCase           = CciString(CONST_UPPERCASE);
  constLowerCase           = CciString(CONST_LOWERCASE);
  constSpace	            = CciString(CONST_SPACE);									/* 22/09/2001 */
}


/******************************************************************************/
/*                                                                            */
/* Plugin Node Implementation Function:                                       */
/*                                                                            */
/* This function is called when the message broker loads the "lil" during the */
/* initialization of the broker engine.                                       */
/*                                                                            */
/* The responsibilities of the plug-in are to:                                */
/*  - create the node factory (or factories) which the "lil" supports, using  */
/*    the createNodeFactory() function, saving the returned pointer to the    */
/*    node factory for use at runtime.                                        */
/*  - define each node supported by the (or each) factory) and pass a pointer */
/*    to a function table which contains pointers to the node implementation  */
/*    functions.                                                              */
/*  - define the name of each attribute supported by each node, including a   */
/*    pointer to a validation function.                                       */
/*  - define input and output terminals supported by each node.               */
/*  - return the pointer to the factory, unless an error occurred, in which   */
/*    case zero should be returned.                                           */    
/*                                                                            */
/******************************************************************************/
CciFactory LilFactoryExportPrefix * LilFactoryExportSuffix *bipGetMessageflowNodeFactory()
{
   /*@unused@*/static char *functionName = (char *)"bipGetMessageflowNodeFactory()";/* 28/07/2001 */
   CciFactory *factoryObject = NULL;
   int rc = 0;

#if defined(CCI_EXCEPTION_ST_DEFAULT)
   CCI_EXCEPTION_ST exception_st = {CCI_EXCEPTION_ST_DEFAULT};
#else
   CCI_EXCEPTION_ST exception_st;
#endif     
   
#if defined(CNI_VFT_DEFAULT)
   static CNI_VFT vftable = {CNI_VFT_DEFAULT};
#else
   static CNI_VFT vftable;

   memset(&vftable, 0, sizeof(vftable));
#endif

   /* Before we proceed we need to initialise all the static constants */
   /* that may be used by the plug-in.                                 */
   initNodeConstants();

   /* Create the Node Factory for this plug-in */
   factoryObject = cniCreateNodeFactory(0, (unsigned short *)constPluginNodeFactory);

   if (factoryObject == CCI_NULL_ADDR) {
      if (rc == CCI_EXCEPTION) {
         /* Get details of the exception */
         memset(&exception_st, 0, sizeof(exception_st));
         cciGetLastExceptionData(&rc, &exception_st);

         /* Any local error handling may go here */

         /* Rethrow the exception */
         cciRethrowLastException(&rc);
      }
      /* Any further local error handling can go here */
   }
   else {
      /* Setup function table with pointers to node implementation functions */
      vftable.iFpCreateNodeContext     = createNodeContext;
      vftable.iFpDeleteNodeContext     = deleteNodeContext;
      vftable.iFpGetAttributeName      = getAttributeName;
      vftable.iFpSetAttribute          = setAttribute;
      vftable.iFpGetAttribute          = getAttribute;
      vftable.iFpEvaluate              = evaluate;
      /* Define a node type supported by our factory. If any errors/exceptions    */
      /* occur during the execution of this utility function, then as we have not */
      /* supplied the returnCode argument, the exception will bypass the plugin   */
      /* and be directly handled by the broker.                                   */
      cniDefineNodeClass(0, factoryObject, (CciChar *)constPlugInNode, &vftable);
   }

   /* Return address of this factory object to the broker */
   return(factoryObject);
}


CciChar* ASCIIToUnicode(const char *aStr)
{
   size_t len = strlen(aStr);
	int rc = -1;

   CciChar *outStr = (CciChar *)malloc(sizeof(CciChar) * (len + 1));
	(void)cciMbsToUcs(&rc, aStr, outStr, len + 1, 1208) ;

   outStr[len] = 0;
   return outStr;
}


CciChar* ASCIInToUnicode(const char* aStr, size_t len)
{
	int rc = -1;
   CciChar *outStr = (CciChar *)malloc(sizeof(CciChar) * (len + 1));

	(void)cciMbsToUcs(&rc, aStr, outStr, len + 1, 1208) ;

   outStr[len] = 0;
   return outStr;
}


void wgetNameValue(char *Env, CciChar **Name, CciChar **Value)
{
   int i = 0;

   while ( Env[i] != '\0' && Env[i] != '=' ) i++;
      
   *Name  = ASCIInToUnicode(Env, i);
   /* 5 Feb 2001 */
   if (Env[i + 1] == '\0')
      *Value = NULL;
   else
      *Value = ASCIIToUnicode(&(Env[i + 1])); 
}


CciChar *wStrUpper(CciChar *string)
{
   int i = 0;
   while ( ( string[i] = (char)towupper(string[i]) ) != 0 )
      i++;
   return(string);
}


CciChar *wStrLower(CciChar *string)
{
   int i = 0;
   while ( ( string[i] = (char)tolower(string[i]) ) != 0 )
      i++;
   return(string);
}

